@@ -11,8 +11,15 @@ configured by the user. Any logged messages before
 Special care is taken by both methods to ensure all errors are logged
 and properly flushed before program exit.
 
+The `logging` module is useful for recording messages about about what
+Certbot is doing under the hood, but do not necessarily need to be shown
+to the user on the terminal. The default verbosity is WARNING.
+
+The preferred method to display important information to the user is to
+use `certbot.display.util` and `certbot.display.ops`.
+
 """
-from __future__ import print_function
+
 
 import functools
 import logging
@@ -21,6 +28,7 @@ import shutil
 import sys
 import tempfile
 import traceback
+from types import TracebackType
 
 from acme import messages
 from certbot import errors
@@ -71,7 +79,9 @@ def pre_arg_parse_setup():
     util.atexit_register(logging.shutdown)
     sys.excepthook = functools.partial(
         pre_arg_parse_except_hook, memory_handler,
-        debug='--debug' in sys.argv, log_path=temp_handler.path)
+        debug='--debug' in sys.argv,
+        quiet='--quiet' in sys.argv or '-q' in sys.argv,
+        log_path=temp_handler.path)
 
 
 def post_arg_parse_setup(config):
@@ -88,7 +98,6 @@ def post_arg_parse_setup(config):
     """
     file_handler, file_path = setup_log_file_handler(
         config, 'letsencrypt.log', FILE_FMT)
-    logs_dir = os.path.dirname(file_path)
 
     root_logger = logging.getLogger()
     memory_handler = stderr_handler = None
@@ -102,22 +111,29 @@ def post_arg_parse_setup(config):
 
     root_logger.addHandler(file_handler)
     root_logger.removeHandler(memory_handler)
-    temp_handler = memory_handler.target  # pylint: disable=no-member
+    temp_handler = getattr(memory_handler, 'target', None)
     memory_handler.setTarget(file_handler)  # pylint: disable=no-member
     memory_handler.flush(force=True)  # pylint: disable=unexpected-keyword-arg
     memory_handler.close()
-    temp_handler.close()
+    if temp_handler:
+        temp_handler.close()
 
     if config.quiet:
         level = constants.QUIET_LOGGING_LEVEL
+    elif config.verbose_level is not None:
+        level = constants.DEFAULT_LOGGING_LEVEL - int(config.verbose_level) * 10
     else:
-        level = -config.verbose_count * 10
+        level = constants.DEFAULT_LOGGING_LEVEL - config.verbose_count * 10
+
     stderr_handler.setLevel(level)
     logger.debug('Root logging level set at %d', level)
-    logger.info('Saving debug log to %s', file_path)
+
+    if not config.quiet:
+        print(f'Saving debug log to {file_path}', file=sys.stderr)
 
     sys.excepthook = functools.partial(
-        post_arg_parse_except_hook, debug=config.debug, log_path=logs_dir)
+        post_arg_parse_except_hook,
+        debug=config.debug, quiet=config.quiet, log_path=file_path)
 
 
 def setup_log_file_handler(config, logfile, fmt):
@@ -163,7 +179,7 @@ class ColoredStreamHandler(logging.StreamHandler):
 
     """
     def __init__(self, stream=None):
-        super(ColoredStreamHandler, self).__init__(stream)
+        super().__init__(stream)
         self.colored = (sys.stderr.isatty() if stream is None else
                         stream.isatty())
         self.red_level = logging.WARNING
@@ -177,7 +193,7 @@ class ColoredStreamHandler(logging.StreamHandler):
         :rtype: str
 
         """
-        out = super(ColoredStreamHandler, self).format(record)
+        out = super().format(record)
         if self.colored and record.levelno >= self.red_level:
             return ''.join((util.ANSI_SGR_RED, out, util.ANSI_SGR_RESET))
         return out
@@ -192,14 +208,14 @@ class MemoryHandler(logging.handlers.MemoryHandler):
     """
     def __init__(self, target=None, capacity=10000):
         # capacity doesn't matter because should_flush() is overridden
-        super(MemoryHandler, self).__init__(capacity, target=target)
+        super().__init__(capacity, target=target)
 
     def close(self):
         """Close the memory handler, but don't set the target to None."""
         # This allows the logging module which may only have a weak
         # reference to the target handler to properly flush and close it.
-        target = self.target
-        super(MemoryHandler, self).close()
+        target = getattr(self, 'target')
+        super().close()
         self.target = target
 
     def flush(self, force=False):  # pylint: disable=arguments-differ
@@ -213,7 +229,7 @@ class MemoryHandler(logging.handlers.MemoryHandler):
         # This method allows flush() calls in logging.shutdown to be a
         # noop so we can control when this handler is flushed.
         if force:
-            super(MemoryHandler, self).flush()
+            super().flush()
 
     def shouldFlush(self, record):
         """Should the buffer be automatically flushed?
@@ -241,7 +257,7 @@ class TempHandler(logging.StreamHandler):
         self._workdir = tempfile.mkdtemp()
         self.path = os.path.join(self._workdir, 'log')
         stream = util.safe_open(self.path, mode='w', chmod=0o600)
-        super(TempHandler, self).__init__(stream)
+        super().__init__(stream)
         self._delete = True
 
     def emit(self, record):
@@ -251,7 +267,7 @@ class TempHandler(logging.StreamHandler):
 
         """
         self._delete = False
-        super(TempHandler, self).emit(record)
+        super().emit(record)
 
     def close(self):
         """Close the handler and the temporary log file.
@@ -267,7 +283,7 @@ class TempHandler(logging.StreamHandler):
             if self._delete:
                 shutil.rmtree(self._workdir)
             self._delete = False
-            super(TempHandler, self).close()
+            super().close()
         finally:
             self.release()
 
@@ -299,7 +315,8 @@ def pre_arg_parse_except_hook(memory_handler, *args, **kwargs):
         memory_handler.flush(force=True)
 
 
-def post_arg_parse_except_hook(exc_type, exc_value, trace, debug, log_path):
+def post_arg_parse_except_hook(exc_type: type, exc_value: BaseException, trace: TracebackType,
+                               debug: bool, quiet: bool, log_path: str):
     """Logs fatal exceptions and reports them to the user.
 
     If debug is True, the full exception and traceback is shown to the
@@ -310,15 +327,21 @@ def post_arg_parse_except_hook(exc_type, exc_value, trace, debug, log_path):
     :param BaseException exc_value: raised exception
     :param traceback trace: traceback of where the exception was raised
     :param bool debug: True if the traceback should be shown to the user
+    :param bool quiet: True if Certbot is running in quiet mode
     :param str log_path: path to file or directory containing the log
 
     """
     exc_info = (exc_type, exc_value, trace)
+    # Only print human advice if not running under --quiet
+    exit_func = lambda: sys.exit(1) if quiet else exit_with_advice(log_path)
     # constants.QUIET_LOGGING_LEVEL or higher should be used to
     # display message the user, otherwise, a lower level like
     # logger.DEBUG should be used
     if debug or not issubclass(exc_type, Exception):
         assert constants.QUIET_LOGGING_LEVEL <= logging.ERROR
+        if exc_type is KeyboardInterrupt:
+            logger.error('Exiting due to user request.')
+            sys.exit(1)
         logger.error('Exiting abnormally:', exc_info=exc_info)
     else:
         logger.debug('Exiting abnormally:', exc_info=exc_info)
@@ -326,7 +349,7 @@ def post_arg_parse_except_hook(exc_type, exc_value, trace, debug, log_path):
         # our logger printing warnings and errors in red text.
         if issubclass(exc_type, errors.Error):
             logger.error(str(exc_value))
-            sys.exit(1)
+            exit_func()
         logger.error('An unexpected error occurred:')
         if messages.is_acme_error(exc_value):
             # Remove the ACME error prefix from the exception
@@ -339,11 +362,11 @@ def post_arg_parse_except_hook(exc_type, exc_value, trace, debug, log_path):
             # and remove the final newline before passing it to
             # logger.error.
             logger.error(''.join(output).rstrip())
-    exit_with_log_path(log_path)
+    exit_func()
 
 
-def exit_with_log_path(log_path):
-    """Print a message about the log location and exit.
+def exit_with_advice(log_path: str):
+    """Print a link to the community forums, the debug log path, and exit
 
     The message is printed to stderr and the program will exit with a
     nonzero status.
@@ -351,10 +374,11 @@ def exit_with_log_path(log_path):
     :param str log_path: path to file or directory containing the log
 
     """
-    msg = 'Please see the '
+    msg = ("Ask for help or search for solutions at https://community.letsencrypt.org. "
+           "See the ")
     if os.path.isdir(log_path):
-        msg += 'logfiles in {0} '.format(log_path)
+        msg += f'logfiles in {log_path} '
     else:
-        msg += "logfile '{0}' ".format(log_path)
-    msg += 'for more details.'
+        msg += f"logfile {log_path} "
+    msg += 'or re-run Certbot with -v for more details.'
     sys.exit(msg)
