@@ -7,6 +7,9 @@ from multiprocessing import Process
 import shutil
 import sys
 import tempfile
+from typing import Iterable
+from typing import List
+from typing import Optional
 import unittest
 import warnings
 
@@ -16,17 +19,21 @@ import josepy as jose
 import OpenSSL
 import pkg_resources
 
-from certbot import interfaces
+from certbot import configuration
 from certbot import util
-from certbot._internal import configuration
 from certbot._internal import constants
 from certbot._internal import lock
 from certbot._internal import storage
+from certbot._internal.display import obj as display_obj
 from certbot.compat import filesystem
 from certbot.compat import os
 from certbot.display import util as display_util
+from certbot.plugins import common
 
 try:
+    # When we remove this deprecated import, we should also remove the
+    # "external-mock" test environment and the mock dependency listed in
+    # tools/pinning/pyproject.toml.
     import mock
     warnings.warn(
         "The external mock module is being used for backwards compatibility "
@@ -34,10 +41,44 @@ try:
         "use unittest.mock. Be sure to update your code accordingly.",
         PendingDeprecationWarning
     )
-except ImportError: # pragma: no cover
-    from unittest import mock # type: ignore
+except ImportError:  # pragma: no cover
+    from unittest import mock  # type: ignore
 
 
+class DummyInstaller(common.Installer):
+    """Dummy installer plugin for test purpose."""
+    def get_all_names(self) -> Iterable[str]:
+        pass
+
+    def deploy_cert(self, domain: str, cert_path: str, key_path: str, chain_path: str,
+                    fullchain_path: str) -> None:
+        pass
+
+    def enhance(self, domain: str, enhancement: str, options: Optional[List[str]] = None) -> None:
+        pass
+
+    def supported_enhancements(self) -> List[str]:
+        pass
+
+    def save(self, title: Optional[str] = None, temporary: bool = False) -> None:
+        pass
+
+    def config_test(self) -> None:
+        pass
+
+    def restart(self) -> None:
+        pass
+
+    @classmethod
+    def add_parser_arguments(cls, add):
+        pass
+
+    def prepare(self) -> None:
+        pass
+
+    def more_info(self) -> str:
+        pass
+
 
 def vector_path(*names):
     """Path to a test vector."""
@@ -147,10 +188,7 @@ def make_lineage(config_dir, testfile, ec=False):
 
 
 def patch_get_utility(target='zope.component.getUtility'):
-    """Patch zope.component.getUtility to use a special mock IDisplay.
-
-    The mock IDisplay works like a regular mock object, except it also
-    also asserts that methods are called with valid arguments.
+    """Deprecated, patch certbot.display.util directly or use patch_display_util instead.
 
     :param str target: path to patch
 
@@ -158,18 +196,16 @@ def patch_get_utility(target='zope.component.getUtility'):
     :rtype: mock.MagicMock
 
     """
-    return mock.patch(target, new_callable=_create_get_utility_mock)
+    warnings.warn('Decorator certbot.tests.util.patch_get_utility is deprecated. You should now '
+                  'patch certbot.display.util yourself directly or use '
+                  'certbot.tests.util.patch_display_util as a temporary workaround.')
+    return mock.patch(target, new_callable=_create_display_util_mock)
 
 
 def patch_get_utility_with_stdout(target='zope.component.getUtility',
                                   stdout=None):
-    """Patch zope.component.getUtility to use a special mock IDisplay.
-
-    The mock IDisplay works like a regular mock object, except it also
-    also asserts that methods are called with valid arguments.
-
-    The `message` argument passed to the IDisplay methods is passed to
-    stdout's write method.
+    """Deprecated, patch certbot.display.util directly
+    or use patch_display_util_with_stdout instead.
 
     :param str target: path to patch
     :param object stdout: object to write standard output to; it is
@@ -178,11 +214,71 @@ def patch_get_utility_with_stdout(target='zope.component.getUtility',
     :returns: mock zope.component.getUtility
     :rtype: mock.MagicMock
 
+    """
+    warnings.warn('Decorator certbot.tests.util.patch_get_utility_with_stdout is deprecated. You '
+                  'should now patch certbot.display.util yourself directly or use '
+                  'use certbot.tests.util.patch_display_util_with_stdout as a temporary '
+                  'workaround.')
+    stdout = stdout if stdout else io.StringIO()
+    freezable_mock = _create_display_util_mock_with_stdout(stdout)
+    return mock.patch(target, new=freezable_mock)
+
+
+def patch_display_util():
+    """Patch certbot.display.util to use a special mock IDisplay.
+
+    The mock IDisplay works like a regular mock object, except it also
+    also asserts that methods are called with valid arguments.
+
+    The mock created by this patch mocks out Certbot internals so this can be
+    used like the old patch_get_utility function. That is, the mock object will
+    be called by the certbot.display.util functions and the mock returned by
+    that call will be used as the IDisplay object. This was done to simplify
+    the transition from zope.component and mocking certbot.display.util
+    functions directly in test code should be preferred over using this
+    function in the future.
+
+    See https://github.com/certbot/certbot/issues/8948
+
+    :returns: patch on the function used internally by certbot.display.util to
+        get an IDisplay object
+    :rtype: unittest.mock._patch
+
+    """
+    return mock.patch('certbot._internal.display.obj.get_display',
+                      new_callable=_create_display_util_mock)
+
+
+def patch_display_util_with_stdout(stdout=None):
+    """Patch certbot.display.util to use a special mock IDisplay.
+
+    The mock IDisplay works like a regular mock object, except it also
+    asserts that methods are called with valid arguments.
+
+    The mock created by this patch mocks out Certbot internals so this can be
+    used like the old patch_get_utility function. That is, the mock object will
+    be called by the certbot.display.util functions and the mock returned by
+    that call will be used as the IDisplay object. This was done to simplify
+    the transition from zope.component and mocking certbot.display.util
+    functions directly in test code should be preferred over using this
+    function in the future.
+
+    See https://github.com/certbot/certbot/issues/8948
+
+    The `message` argument passed to the IDisplay methods is passed to
+    stdout's write method.
+
+    :param object stdout: object to write standard output to; it is
+        expected to have a `write` method
+    :returns: patch on the function used internally by certbot.display.util to
+        get an IDisplay object
+    :rtype: unittest.mock._patch
+
     """
     stdout = stdout if stdout else io.StringIO()
 
-    freezable_mock = _create_get_utility_mock_with_stdout(stdout)
-    return mock.patch(target, new=freezable_mock)
+    return mock.patch('certbot._internal.display.obj.get_display',
+                      new=_create_display_util_mock_with_stdout(stdout))
 
 
 class FreezableMock:
@@ -253,18 +349,21 @@ class FreezableMock:
         return object.__setattr__(self, name, value)
 
 
-def _create_get_utility_mock():
+def _create_display_util_mock():
     display = FreezableMock()
     # Use pylint code for disable to keep on single line under line length limit
-    for name in interfaces.IDisplay.names():  # pylint: E1120
-        if name != 'notification':
+    method_list = [func for func in dir(display_obj.FileDisplay)
+                   if callable(getattr(display_obj.FileDisplay, func))
+                   and not func.startswith("__")]
+    for method in method_list:
+        if method != 'notification':
             frozen_mock = FreezableMock(frozen=True, func=_assert_valid_call)
-            setattr(display, name, frozen_mock)
+            setattr(display, method, frozen_mock)
     display.freeze()
     return FreezableMock(frozen=True, return_value=display)
 
 
-def _create_get_utility_mock_with_stdout(stdout):
+def _create_display_util_mock_with_stdout(stdout):
     def _write_msg(message, *unused_args, **unused_kwargs):
         """Write to message to stdout.
         """
@@ -278,20 +377,20 @@ def _create_get_utility_mock_with_stdout(stdout):
         _assert_valid_call(args, kwargs)
         _write_msg(*args, **kwargs)
 
-
     display = FreezableMock()
     # Use pylint code for disable to keep on single line under line length limit
-    for name in interfaces.IDisplay.names():  # pylint: E1120
-        if name == 'notification':
+    method_list = [func for func in dir(display_obj.FileDisplay)
+                   if callable(getattr(display_obj.FileDisplay, func))
+                   and not func.startswith("__")]
+    for method in method_list:
+        if method == 'notification':
             frozen_mock = FreezableMock(frozen=True,
                                         func=_write_msg)
-            setattr(display, name, frozen_mock)
         else:
             frozen_mock = FreezableMock(frozen=True,
                                         func=mock_method)
-            setattr(display, name, frozen_mock)
+        setattr(display, method, frozen_mock)
     display.freeze()
-
     return FreezableMock(frozen=True, return_value=display)
 
 
@@ -331,18 +430,18 @@ class TempDirTestCase(unittest.TestCase):
 class ConfigTestCase(TempDirTestCase):
     """Test class which sets up a NamespaceConfig object."""
     def setUp(self):
-        super(ConfigTestCase, self).setUp()
+        super().setUp()
         self.config = configuration.NamespaceConfig(
             mock.MagicMock(**constants.CLI_DEFAULTS)
         )
-        self.config.verb = "certonly"
-        self.config.config_dir = os.path.join(self.tempdir, 'config')
-        self.config.work_dir = os.path.join(self.tempdir, 'work')
-        self.config.logs_dir = os.path.join(self.tempdir, 'logs')
-        self.config.cert_path = constants.CLI_DEFAULTS['auth_cert_path']
-        self.config.fullchain_path = constants.CLI_DEFAULTS['auth_chain_path']
-        self.config.chain_path = constants.CLI_DEFAULTS['auth_chain_path']
-        self.config.server = "https://example.com"
+        self.config.namespace.verb = "certonly"
+        self.config.namespace.config_dir = os.path.join(self.tempdir, 'config')
+        self.config.namespace.work_dir = os.path.join(self.tempdir, 'work')
+        self.config.namespace.logs_dir = os.path.join(self.tempdir, 'logs')
+        self.config.namespace.cert_path = constants.CLI_DEFAULTS['auth_cert_path']
+        self.config.namespace.fullchain_path = constants.CLI_DEFAULTS['auth_chain_path']
+        self.config.namespace.chain_path = constants.CLI_DEFAULTS['auth_chain_path']
+        self.config.namespace.server = "https://example.com"
 
 
 def _handle_lock(event_in, event_out, path):
